
package net.sourceforge.guacamole.net.auth.ldap;

/*
 *  Guacamole - Clientless Remote Desktop
 *  Copyright (C) 2010  Michael Jumper
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Affero General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Affero General Public License for more details.
 *
 *  You should have received a copy of the GNU Affero General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import com.novell.ldap.LDAPConnection;
import com.novell.ldap.LDAPEntry;
import com.novell.ldap.LDAPException;
import com.novell.ldap.LDAPSearchResults;
import java.io.UnsupportedEncodingException;
import java.util.Enumeration;
import net.sourceforge.guacamole.net.auth.AuthenticationProvider;
import java.util.HashMap;
import java.util.Map;
import net.sourceforge.guacamole.GuacamoleException;
import net.sourceforge.guacamole.net.auth.UsernamePassword;
import net.sourceforge.guacamole.net.auth.ldap.properties.LDAPGuacamoleProperties;
import net.sourceforge.guacamole.properties.GuacamoleProperties;
import net.sourceforge.guacamole.protocol.GuacamoleConfiguration;


public class LDAPAuthenticationProvider implements AuthenticationProvider<UsernamePassword> {

    @Override
    public Map<String, GuacamoleConfiguration> getAuthorizedConfigurations(UsernamePassword credentials) throws GuacamoleException {

        try {
           
            // Connect to LDAP server
            LDAPConnection ldapConnection = new LDAPConnection();
            ldapConnection.connect(
                    GuacamoleProperties.getProperty(LDAPGuacamoleProperties.LDAP_HOSTNAME),
                    GuacamoleProperties.getProperty(LDAPGuacamoleProperties.LDAP_PORT)
            );

            // Construct user DN
            String user_dn = 
                "cn=" + credentials.getUsername()
                + "," + GuacamoleProperties.getProperty(LDAPGuacamoleProperties.LDAP_SUFFIX);
            
            // Bind as user 
            try {
                ldapConnection.bind(
                        LDAPConnection.LDAP_V3,
                        user_dn,
                        credentials.getPassword().getBytes("UTF-8") 
                );
            }
            catch (UnsupportedEncodingException e) {
                throw new GuacamoleException(e);
            }

            // Find all guac configs for this user
            LDAPSearchResults results = ldapConnection.search(
                    user_dn,
                    LDAPConnection.SCOPE_ONE,
                    "(objectClass=guacConfig)",
                    null,
                    false
            );

            // Add all configs
            Map<String, GuacamoleConfiguration> configs = new HashMap<String, GuacamoleConfiguration>();
            while (results.hasMore()) {
                
                LDAPEntry entry = results.next();

                String config_dn = entry.getDN();
                GuacamoleConfiguration config = new GuacamoleConfiguration();

                config.setProtocol(entry.getAttribute("protocol").getStringValue());
                Enumeration<String> parameters = entry.getAttribute("parameter").getStringValues();

                while (parameters.hasMoreElements()) {

                    String parameter = parameters.nextElement();

                    // Parse parameter
                    int equals = parameter.indexOf('=');
                    if (equals != -1) {

                        // Parse name
                        String name = parameter.substring(0, equals);
                        String value = parameter.substring(equals+1);

                        config.setParameter(name, value);
                        
                    }
                    
                }
                
                configs.put(config_dn, config);

            }

            // Disconnect
            ldapConnection.disconnect();
            return configs;
            
        }
        catch (LDAPException e) {
            throw new GuacamoleException(e);
        }
        

    }

}

